/*
** Copyright (C) 1991, 1992 by
** Digital Equipment Corporation, Maynard, Mass.
**
** This software is furnished under a license and may be used and copied
** only  in  accordance  with  the  terms  of such  license and with the
** inclusion of the above copyright notice. This software or  any  other
** copies thereof may not be provided or otherwise made available to any
** other person. No title to and ownership of  the  software  is  hereby
** transferred.
**
** The information in this software is subject to change without  notice
** and  should  not be  construed  as  a commitment by Digital Equipment
** Corporation.
**
** Digital assumes no responsibility for the use or  reliability  of its
** software on equipment which is not supplied by Digital.
**
*/

#include "sys/types.h"                          
#include "config.h"
#include "sys/dce_space.h"

/*
** Enable/disable diagnostic messages. Allowed values are:
**    DCE_TRUE  - enable debug message
**    DCE_FALSE - disable debug message
*/
#define DCE_DEBUG      DCE_FALSE

/* 
** Highest vector number supported. Needed to allocate large enough array 
** Should NOT be modified
*/
#define DCE_MAX_VECTOR_NUMS            16

/* 
** Number of buffers pre-allocated on the receive list
*/
#define DCE_RX_BUFS_TO_POST            4

/* 
** EISA Slot ID value for DC21040 adapter 
*/
#define DCE_PLX_EISA_ID_VAL            0x5042A310    

/* 
** PCI CFID value for DC21040 adapter 
*/
#define DCE_PCI_CFID_VAL               0x00021011  /* 0002 - 21040 device ID
                                                   ** 1011 - DEC vendor ID
                                                   */

/* 
** # of packets queued to read queues before rx processing is disabled
** (used as a high-water mark to disable driver rx processing of
** packets from the adapter)
*/
#define DCE_RX_PKTS_QUEUED_THRESH_HI   16

/* 
** When less than this # of packets are queued to read queue then
** driver receive processing will be re-enabled if it was previously
** disabled.         
*/
#define DCE_RX_PKTS_QUEUED_THRESH_LO   8


/* 
** Maximum number of streams connected (opened) to each adapter .
** Revision 1 adapters are limited to 16. If the number defined here is
** greater than the limit supported by the adapter's revision, or greater
** than the maximum supported by the driver, the driver will use the minimum
** of all the limitations
*/
#define DCE_MAX_USRS                   16

/* 
** When enabled, frames sent by a user will be looped back to the user
** if the frame meets the perfect filter criteria for that user.
** Default is enable.
*/
#define DCE_UNIFORM_CARD_MODEL         DCE_TRUE

/* 
** SIA mode. Can have one of the following values:
**     DCE_TP_FULL_DUPLEX_MODE         - point to point full duplex twisted 
**                                       pair connection
**     DCE_AUI_MODE                    - AUI connection (default)
**     DCE_BNC_MODE                    - BNC connection 
**     DCE_TP_MODE                     - twisted pair connection
**     DCE_AUTODETECT_MODE             - Automatic detection between TP BNC 
**                                       and AUI
**     DCE_TP_IGNORE_LINKTEST_POS_MODE - Twisted pair connection with linkfail
**                                       test ignore & set polarity plus
**     DCE_TP_IGNORE_LINKTEST_NEG_MODE - Twisted pair connection with linkfail
**                                       test ignore 
** 
** Important Note!
** This definition is a global definition for all installed boards. 
** If you want to set a different mode for each board, please modify 
** the 'sia_mode' field in the appropriate part of the DCE_info
** structure below (the values to use are those listed above).
*/
#define DCE_SIA_MODE          DCE_AUTODETECT_MODE

/*
** Selects internal bus arbitration between the receive and transmit
** processing. Allowed values:
**    DCE_TRUE  - round robin arbitration, equal sharing
**    DCE_FALSE - receive process has priority over the transmit process
**                (default)
*/
#define DCE_BUS_ARBITRATION   DCE_FALSE

/*
** Maximum number of longwords transferred in a DMA transaction.
** Allowed values are: 0, 1, 2, 4, 8, 16 (default), 32. Burst length of 0
** means the only limit is the amount of data stored.
** Burst length of 32 is not supported in Revision 1 boards.
*/
#define DCE_BURST_LENGTH      16

/*
** Address boundary for data burst stop. Allowed values are:
**    DCE_CACHE_8
**    DCE_CACHE_16 (default)
**    DCE_CACHE_32
*/
#define DCE_CACHE_ALIGNMENT   DCE_CACHE_16

/*
** Enable/disable passing of bad frames (runt frames, truncated frames,
** collided frames) from the adapter to the driver.
**    DCE_TRUE  - enables passing of bad frames
**    DCE_FALSE - disable passingof bad frames (default)
*/
#define DCE_PASS_BAD_FRAMES   DCE_FALSE 

/*
** Backoff counter algorithm.
**    DCE_TRUE  - stop counter when any carrier activity is detected
**    DCE_FALSE - backoff counter is not affected by the carrier activity
**                (default)
*/
#define DCE_BACKOFF_CNTR      DCE_FALSE

/*
** Transmit FIFO threshold for starting packet transmission to line.
** Allowed values are:
**    DCE_THRESHOLD_72
**    DCE_THRESHOLD_96 
**    DCE_THRESHOLD_128
**    DCE_THRESHOLD_160 (default)
*/
#define DCE_THRESHOLD         DCE_THRESHOLD_160

/*
** Enable/disable backpressure algorithm.
**    DCE_TRUE  - enable backpressure algorithm
**    DCE_FALSE - disable backpressure algorithm (default)
*/
#define DCE_BACK_PRESSURE     DCE_FALSE

/*
** Enable/disable capture effect algorithm.
**    DCE_TRUE  - enable capture effect algorithm
**    DCE_FALSE - disable capture effect algorithm (default)
*/
#define DCE_CAPTURE_EFFECT_ENABLE DCE_FALSE

/*
** Enable/disable parity checking on PCI bus
**    DCE_TRUE  - enable parity checking
**    DCE_FALSE - disable parity checking (default)
*/
#define DCE_PARITY_ERROR_RESPONSE DCE_FALSE

/*
** DC21040 PCI Latency Timer, in units of PCI bus clocks. Default is 18.
*/
#define DCE_LATENCY_TIMER     18
/*
** Enable/Disable SW's CRC calculation (in transmit packets)
**    DCE_SW_CRC_ALWAYS - CRC will be calculated by the software for every 
**                        transmit packet longer than the thershold, for
**                        transmit packets shorter than the threshold the HW
**                        will calculate the CRC
**    DCE_SW_CRC_NEVER  - CRC will be calculated by the hardware for every 
**                        transmit packet 
*/
#define DCE_CRC_CALC          DCE_SW_CRC_ALWAYS

/*
#ifdef DCE_CNTLS
#define DCE_MAX_BOARDS        DCE_CNTLS
#else
#ifdef DCE_0
#define DCE_MAX_BOARDS        1
#endif
#ifdef DCE1_0
#define DCE_MAX_BOARDS        2
#endif
#ifdef DCE2_0
#define DCE_MAX_BOARDS        3
#endif
#ifdef DCE3_0
#define DCE_MAX_BOARDS        4
#endif
#endif
*/

#ifdef DCE3_0
#define DCE_MAX_BOARDS        4
#else
#ifdef DCE2_0
#define DCE_MAX_BOARDS        3
#else
#ifdef DCE1_0
#define DCE_MAX_BOARDS        2
#else
#ifdef DCE_0
#define DCE_MAX_BOARDS        1
#endif
#endif
#endif
#endif


DCE_board_config_t DCE_info[DCE_MAX_BOARDS] =
{
#ifdef DCE_0
   {
   0,                            /* board_num */
   "dce0",                       /* board_id */
   DCE_0_VECT,                   /* vector */
   DCE_0_SIOA,                   /* iobase */
   DCE_0_EIOA,                   /* ioend */
   DCE_0_SIOA,                   /* placeholder for cfgbase */
   DCE_PLX_EISA_ID_VAL,          /* probe_val*/            
   DCE_PCI_CFID_VAL,             /* cfid_probe_val*/
#ifdef DCE_CMAJOR_0
   DCE_CMAJOR_0,                 /* major_num */
#else
   -1,
#endif
   DCE_RX_BUFS_TO_POST,          /* rx_bufs_to_post */
   DCE_RX_PKTS_QUEUED_THRESH_HI, /* rx_pkts_queued_thresh_hi */
   DCE_RX_PKTS_QUEUED_THRESH_LO, /* rx_pkts_queued_thresh_lo */
   DCE_UNIFORM_CARD_MODEL,       /* uniform_card_model */
   DCE_SIA_MODE,                 /* sia_mode */
   DCE_BUS_ARBITRATION,          /* bus_arbitration */
   DCE_BURST_LENGTH,             /* burst_length */
   DCE_CACHE_ALIGNMENT,          /* cache_alignment */
   DCE_PASS_BAD_FRAMES,          /* pass_bad_frames */
   DCE_BACKOFF_CNTR,             /* backoff_cntr */
   DCE_THRESHOLD,                /* threshold */
   DCE_BACK_PRESSURE,            /* back_pressure */
   DCE_CAPTURE_EFFECT_ENABLE,    /* capture effect enable */
   DCE_PARITY_ERROR_RESPONSE,    /* parity_error_response */
   DCE_LATENCY_TIMER,            /* latency_timer */
   DCE_NULL_PCI_HANDLER,         /* pci handler place-holder */
   DCE_CRC_CALC                  /* SW/HW CRC calculation */
   }
#endif
#ifdef DCE1_0
   ,{
   1,                            /* board_num */
   "dce1",                       /* board_id */
   DCE1_0_VECT,                  /* vector */
   DCE1_0_SIOA,                  /* iobase */
   DCE1_0_EIOA,                  /* ioend */
   DCE1_0_SIOA,                  /* placeholder for cfgbase */
   DCE_PLX_EISA_ID_VAL,          /* probe_val*/            
   DCE_PCI_CFID_VAL,             /* cfid_probe_val*/
#ifdef DCE_CMAJOR_1
   DCE_CMAJOR_1,                 /* major_num */
#else
   -1,
#endif
   DCE_RX_BUFS_TO_POST,          /* rx_bufs_to_post */
   DCE_RX_PKTS_QUEUED_THRESH_HI, /* rx_pkts_queued_thresh_hi */
   DCE_RX_PKTS_QUEUED_THRESH_LO, /* rx_pkts_queued_thresh_lo */
   DCE_UNIFORM_CARD_MODEL,       /* uniform_card_model */
   DCE_SIA_MODE,                 /* sia_mode */
   DCE_BUS_ARBITRATION,          /* bus_arbitration */
   DCE_BURST_LENGTH,             /* burst_length */
   DCE_CACHE_ALIGNMENT,          /* cache_alignment */
   DCE_PASS_BAD_FRAMES,          /* pass_bad_frames */
   DCE_BACKOFF_CNTR,             /* backoff_cntr */
   DCE_THRESHOLD,                /* threshold */
   DCE_BACK_PRESSURE,            /* back_pressure */
   DCE_CAPTURE_EFFECT_ENABLE,    /* capture effect enable */
   DCE_PARITY_ERROR_RESPONSE,    /* parity_error_response */
   DCE_LATENCY_TIMER,            /* latency_timer */
   DCE_NULL_PCI_HANDLER,         /* pci handler place-holder */
   DCE_CRC_CALC                  /* SW/HW CRC calculation */
   } 
#endif
#ifdef DCE2_0
   ,{
   2,                            /* board_num */
   "dce2",                       /* board_id */
   DCE2_0_VECT,                  /* vector */
   DCE2_0_SIOA,                  /* iobase */
   DCE2_0_EIOA,                  /* ioend */
   DCE2_0_SIOA,                  /* placeholder for cfgbase */
   DCE_PLX_EISA_ID_VAL,          /* probe_val*/            
   DCE_PCI_CFID_VAL,             /* cfid_probe_val*/
#ifdef DCE_CMAJOR_2
   DCE_CMAJOR_2,                 /* major_num */
#else
   -1,
#endif
   DCE_RX_BUFS_TO_POST,          /* rx_bufs_to_post */
   DCE_RX_PKTS_QUEUED_THRESH_HI, /* rx_pkts_queued_thresh_hi */
   DCE_RX_PKTS_QUEUED_THRESH_LO, /* rx_pkts_queued_thresh_lo */
   DCE_UNIFORM_CARD_MODEL,       /* uniform_card_model */
   DCE_SIA_MODE,                 /* sia_mode */
   DCE_BUS_ARBITRATION,          /* bus_arbitration */
   DCE_BURST_LENGTH,             /* burst_length */
   DCE_CACHE_ALIGNMENT,          /* cache_alignment */
   DCE_PASS_BAD_FRAMES,          /* pass_bad_frames */
   DCE_BACKOFF_CNTR,             /* backoff_cntr */
   DCE_THRESHOLD,                /* threshold */
   DCE_BACK_PRESSURE,            /* back_pressure */
   DCE_CAPTURE_EFFECT_ENABLE,    /* capture effect enable */
   DCE_PARITY_ERROR_RESPONSE,    /* parity_error_response */
   DCE_LATENCY_TIMER,            /* latency_timer */
   DCE_NULL_PCI_HANDLER,         /* pci handler place-holder */
   DCE_CRC_CALC                  /* SW/HW CRC calculation */
   }
#endif
#ifdef DCE3_0
   ,{
   3,                            /* board_num */
   "dce3",                       /* board_id */
   DCE3_0_VECT,                  /* vector */
   DCE3_0_SIOA,                  /* iobase */
   DCE3_0_EIOA,                  /* ioend */
   DCE3_0_SIOA,                  /* placeholder for cfgbase */
   DCE_PLX_EISA_ID_VAL,          /* probe_val*/            
   DCE_PCI_CFID_VAL,             /* cfid_probe_val*/
#ifdef DCE_CMAJOR_3
   DCE_CMAJOR_3,                 /* major_num */
#else
   -1,
#endif
   DCE_RX_BUFS_TO_POST,          /* rx_bufs_to_post */
   DCE_RX_PKTS_QUEUED_THRESH_HI, /* rx_pkts_queued_thresh_hi */
   DCE_RX_PKTS_QUEUED_THRESH_LO, /* rx_pkts_queued_thresh_lo */
   DCE_UNIFORM_CARD_MODEL,       /* uniform_card_model */
   DCE_SIA_MODE,                 /* sia_mode */
   DCE_BUS_ARBITRATION,          /* bus_arbitration */
   DCE_BURST_LENGTH,             /* burst_length */
   DCE_CACHE_ALIGNMENT,          /* cache_alignment */
   DCE_PASS_BAD_FRAMES,          /* pass_bad_frames */
   DCE_BACKOFF_CNTR,             /* backoff_cntr */
   DCE_THRESHOLD,                /* threshold */
   DCE_BACK_PRESSURE,            /* back_pressure */
   DCE_CAPTURE_EFFECT_ENABLE,    /* capture effect enable */
   DCE_PARITY_ERROR_RESPONSE,    /* parity_error_response */
   DCE_LATENCY_TIMER,            /* latency_timer */
   DCE_NULL_PCI_HANDLER,         /* pci handler place-holder */
   DCE_CRC_CALC                  /* SW/HW CRC calculation */
   } 
#endif
};

#ifndef TCP_0
   void *ifstats = (void *)0;
#endif


char     *DCE_vector_to_board[DCE_MAX_VECTOR_NUMS];
unchar   DCE_debug      = DCE_DEBUG;
uint     DCE_max_boards = DCE_MAX_BOARDS;
uint     DCE_max_usrs   = DCE_MAX_USRS;
void     *DCE_Boards_table[DCE_MAX_BOARDS];
